exports.cliVersion = ">=3.2";
exports.init = function(logger, config, cli, nodeappc) {
	var path = require('path'),
		foundModules = [];

	cli.on('cli:pre-execute', function (data) {
		if(cli.argv['$command'] !== 'build' || parseFloat(cli.version) < 3.5 || parseFloat(cli.argv.sdk) < 3.5) {
			return;
		}
		if(!cli.tiapp.modules || !cli.tiapp.modules.length) {
			// no modules in use
			logger.debug('No modules found. Skipping architecture verification.');
			return;
		}
		nodeappc.timodule.detect(path.resolve(cli.argv['project-dir']), logger, function(obj) {
			cli.tiapp.modules.forEach(function(element, index, array) {
				if(element.platform.indexOf(cli.argv.platform) !== -1) {
					// Found a module for this platform, look it up in
					if (obj && obj.project && obj.project[cli.argv.platform] && obj.project[cli.argv.platform][element.id]) {
						foundModules.push({
							id: element.id,
							targetVersion: element.version || -1,
							details: obj.project[cli.argv.platform][element.id]
						});
					} else if (obj && obj.global && obj.global[cli.argv.platform] && obj.global[cli.argv.platform][element.id]) {
						foundModules.push({
							id: element.id,
							targetVersion: element.version || -1,
							details: obj.global[cli.argv.platform][element.id]
						});
					}
				}
			});
			parseResults(cli, logger, foundModules);
		});
	});
};

function parseResults(cli, logger, foundModules) {
	var badArch = [],
		maxVer,
		NONE_SPECIFIED = 'does not contain architectures: arm64 or x86_64';
	foundModules.forEach(function(element, index, array) {
		if(element.targetVersion !== -1) {
			if(!element.details[element.targetVersion].manifest || !element.details[element.targetVersion].manifest.architectures) {
				badArch.push({
					id: element.id,
					version: element.targetVersion,
					architectures: NONE_SPECIFIED
				});
			} else if(element.details[element.targetVersion].manifest && element.details[element.targetVersion].manifest.architectures) {
				if(element.details[element.targetVersion].manifest.architectures.indexOf('64') === -1) {
					badArch.push({
						id: element.id,
						version: element.targetVersion,
						architectures: element.details[element.targetVersion].manifest.architectures
					});
				}
			}
		} else {
			maxVer = Object.keys(element.details).sort().reverse()[0];
			if(missing64BitArchitectures(cli.argv.platform, element.details[maxVer].manifest)) {
				badArch.push({
					id: element.id,
					version: (element.targetVersion !== -1) ? element.targetVersion : maxVer,
					architectures: (element.details[maxVer].manifest) ? element.details[maxVer].manifest.architectures : NONE_SPECIFIED
				});
			}
		}
	});
	if(badArch.length > 0) {
		reportResults(cli, logger, badArch);
	}
}

function missing64BitArchitectures(platform, manifest) {
	if(platform !== 'iphone') {
		// only testing iOS modules for now
		return false;
	}
	if(!manifest || !manifest.architectures || manifest.architectures.indexOf('64') === -1) {
		return true;
	}
	return false;
}

function reportResults(cli, logger, badArch) {
	logger.error('Module architecture does not match build target architecture.');
	badArch.forEach(function(element) {
		logger.error('   ' + element.id + ', version ' + element.version + ' ' + element.architectures);
	});
	//logger.info('');
	//logger.info('Bypass the module architecture check by adding --ignore-architecture to the command');
	process.exit();
 }